@@ -20,6 +20,7 @@ from xonsh.history.main import history_main, _xh_parse_args, construct_history
 
 
 CMDS = ["ls", "cat hello kitty", "abc", "def", "touch me", "grep from me"]
+IGNORE_OPTS = ",".join(["ignoredups", "ignoreerr", "ignorespace"])
 
 
 @pytest.fixture
@@ -94,12 +95,13 @@ def test_hist_flush_with_hist_control(hist, xonsh_builtins):
     """Verify explicit flushing of the history works."""
     hf = hist.flush()
     assert hf is None
-    xonsh_builtins.__xonsh__.env["HISTCONTROL"] = "ignoredups,ignoreerr"
+    xonsh_builtins.__xonsh__.env["HISTCONTROL"] = IGNORE_OPTS
     hist.append({"inp": "ls foo1", "rtn": 0})
     hist.append({"inp": "ls foo1", "rtn": 1})
     hist.append({"inp": "ls foo1", "rtn": 0})
     hist.append({"inp": "ls foo2", "rtn": 2})
     hist.append({"inp": "ls foo3", "rtn": 0})
+    hist.append({"inp": "ls secret", "rtn": 0, "spc": True})
     hf = hist.flush()
     assert hf is not None
     while hf.is_alive():
@@ -163,9 +165,9 @@ def test_show_cmd_numerate(inp, commands, offset, hist, xonsh_builtins, capsys):
 
 
 def test_histcontrol(hist, xonsh_builtins):
-    """Test HISTCONTROL=ignoredups,ignoreerr"""
+    """Test HISTCONTROL=ignoredups,ignoreerr,ignorespacee"""
 
-    xonsh_builtins.__xonsh__.env["HISTCONTROL"] = "ignoredups,ignoreerr"
+    xonsh_builtins.__xonsh__.env["HISTCONTROL"] = IGNORE_OPTS
     assert len(hist.buffer) == 0
 
     # An error, buffer remains empty
@@ -236,6 +238,20 @@ def test_histcontrol(hist, xonsh_builtins):
     assert hist.rtns[-1] == -1
     assert hist.inps[-1] == "ls bazz"
 
+    # Success
+    hist.append({"inp": "echo not secret", "rtn": 0, "spc": False})
+    assert len(hist.buffer) == 10
+    assert "echo not secret" == hist.buffer[-1]["inp"]
+    assert 0 == hist.buffer[-1]["rtn"]
+    assert hist.rtns[-1] == 0
+    assert hist.inps[-1] == "echo not secret"
+
+    # Space
+    hist.append({"inp": "echo secret command", "rtn": 0, "spc": True})
+    assert len(hist.buffer) == 10
+    assert hist.rtns[-1] == 0
+    assert hist.inps[-1] == "echo not secret"
+
 
 @pytest.mark.parametrize("args", ["-h", "--help", "show -h", "show --help"])
 def test_parse_args_help(args, capsys):
@@ -475,7 +491,7 @@ def test__xhj_gc_xx_to_rmfiles(
     assert act_files == exp_files
 
     # comparing age is approximate, because xhj_gc_seconds_to_rmfiles computes 'now' on each call.
-    # For test runs, accept anything in the same hour, test cases not that close.  
+    # For test runs, accept anything in the same hour, test cases not that close.
     # We find multi-minute variations in CI environments.
     # This should cover some amount of think time sitting at a breakpoint, too.
     if fn == _xhj_gc_seconds_to_rmfiles:
@@ -483,3 +499,65 @@ def test__xhj_gc_xx_to_rmfiles(
         assert minute_diff <= 60
     else:
         assert act_size == exp_size
+
+
+def test_hist_clear_cmd(hist, xonsh_builtins, capsys, tmpdir):
+    """Verify that the CLI history clear command works."""
+    xonsh_builtins.__xonsh__.env.update({"XONSH_DATA_DIR": str(tmpdir)})
+    xonsh_builtins.__xonsh__.history = hist
+    xonsh_builtins.__xonsh__.env["HISTCONTROL"] = set()
+
+    for ts, cmd in enumerate(CMDS):  # populate the shell history
+        hist.append({"inp": cmd, "rtn": 0, "ts": (ts + 1, ts + 1.5)})
+    assert len(xonsh_builtins.__xonsh__.history) == 6
+
+    history_main(["clear"])
+
+    out, err = capsys.readouterr()
+    assert err.rstrip() == "History cleared"
+    assert len(xonsh_builtins.__xonsh__.history) == 0
+
+
+def test_hist_off_cmd(hist, xonsh_builtins, capsys, tmpdir):
+    """Verify that the CLI history off command works."""
+    xonsh_builtins.__xonsh__.env.update({"XONSH_DATA_DIR": str(tmpdir)})
+    xonsh_builtins.__xonsh__.history = hist
+    xonsh_builtins.__xonsh__.env["HISTCONTROL"] = set()
+
+    for ts, cmd in enumerate(CMDS):  # populate the shell history
+        hist.append({"inp": cmd, "rtn": 0, "ts": (ts + 1, ts + 1.5)})
+    assert len(xonsh_builtins.__xonsh__.history) == 6
+
+    history_main(["off"])
+
+    out, err = capsys.readouterr()
+    assert err.rstrip() == "History off"
+    assert len(xonsh_builtins.__xonsh__.history) == 0
+
+    for ts, cmd in enumerate(CMDS):  # attempt to populate the shell history
+        hist.append({"inp": cmd, "rtn": 0, "ts": (ts + 1, ts + 1.5)})
+
+    assert len(xonsh_builtins.__xonsh__.history) == 0
+
+
+def test_hist_on_cmd(hist, xonsh_builtins, capsys, tmpdir):
+    """Verify that the CLI history on command works."""
+    xonsh_builtins.__xonsh__.env.update({"XONSH_DATA_DIR": str(tmpdir)})
+    xonsh_builtins.__xonsh__.history = hist
+    xonsh_builtins.__xonsh__.env["HISTCONTROL"] = set()
+
+    for ts, cmd in enumerate(CMDS):  # populate the shell history
+        hist.append({"inp": cmd, "rtn": 0, "ts": (ts + 1, ts + 1.5)})
+    assert len(xonsh_builtins.__xonsh__.history) == 6
+
+    history_main(["off"])
+    history_main(["on"])
+
+    out, err = capsys.readouterr()
+    assert err.rstrip().endswith("History on")
+    assert len(xonsh_builtins.__xonsh__.history) == 0
+
+    for ts, cmd in enumerate(CMDS):  # populate the shell history
+        hist.append({"inp": cmd, "rtn": 0, "ts": (ts + 1, ts + 1.5)})
+
+    assert len(xonsh_builtins.__xonsh__.history) == 6
