@@ -1,12 +1,13 @@
+"""Tests involving running Xonsh in subproc.
+This requires Xonsh installed in venv or otherwise available on PATH
+"""
 import os
-import sys
 import shutil
 import tempfile
 import subprocess as sp
 
 import pytest
 
-import xonsh
 from xonsh.lib.os import indir
 
 from tools import (
@@ -19,50 +20,34 @@ from tools import (
 )
 
 
-XONSH_PREFIX = xonsh.__file__
-if "site-packages" in XONSH_PREFIX:
-    # must be installed version of xonsh
-    num_up = 5
-else:
-    # must be in source dir
-    num_up = 2
-for i in range(num_up):
-    XONSH_PREFIX = os.path.dirname(XONSH_PREFIX)
 PATH = (
-    os.path.join(os.path.dirname(__file__), "bin")
-    + os.pathsep
-    + os.path.join(XONSH_PREFIX, "bin")
-    + os.pathsep
-    + os.path.join(XONSH_PREFIX, "Scripts")
-    + os.pathsep
-    + os.path.join(XONSH_PREFIX, "scripts")
-    + os.pathsep
-    + os.path.dirname(sys.executable)
+    os.path.join(os.path.abspath(os.path.dirname(__file__)), "bin")
     + os.pathsep
     + os.environ["PATH"]
 )
 
 
 skip_if_no_xonsh = pytest.mark.skipif(
-    shutil.which("xonsh", path=PATH) is None, reason="xonsh not on PATH"
+    shutil.which("xonsh") is None, reason="xonsh not on PATH"
 )
 skip_if_no_make = pytest.mark.skipif(
-    shutil.which("make", path=PATH) is None, reason="make command not on PATH"
+    shutil.which("make") is None, reason="make command not on PATH"
 )
 skip_if_no_sleep = pytest.mark.skipif(
-    shutil.which("sleep", path=PATH) is None, reason="sleep command not on PATH"
+    shutil.which("sleep") is None, reason="sleep command not on PATH"
 )
 
 
-def run_xonsh(cmd, stdin=sp.PIPE, stdout=sp.PIPE, stderr=sp.STDOUT, single_command=False):
+def run_xonsh(
+    cmd, stdin=sp.PIPE, stdout=sp.PIPE, stderr=sp.STDOUT, single_command=False
+):
     env = dict(os.environ)
     env["PATH"] = PATH
-    env["XONSH_DEBUG"] = "0" # was "1"
+    env["XONSH_DEBUG"] = "0"  # was "1"
     env["XONSH_SHOW_TRACEBACK"] = "1"
     env["RAISE_SUBPROC_ERROR"] = "0"
     env["PROMPT"] = ""
-    xonsh = "xonsh.bat" if ON_WINDOWS else "xon.sh"
-    xonsh = shutil.which(xonsh, path=PATH)
+    xonsh = shutil.which("xonsh", path=PATH)
     if single_command:
         args = [xonsh, "--no-rc", "-c", cmd]
         input = None
@@ -80,7 +65,7 @@ def run_xonsh(cmd, stdin=sp.PIPE, stdout=sp.PIPE, stderr=sp.STDOUT, single_comma
     )
 
     try:
-        out, err = proc.communicate(input=input, timeout=10)
+        out, err = proc.communicate(input=input, timeout=20)
     except sp.TimeoutExpired:
         proc.kill()
         raise
@@ -227,9 +212,9 @@ g
 with open('tttt', 'w') as fp:
     fp.write("Wow mom!\\n")
 
-![cat tttt | wc]
+![python tests/bin/cat tttt | python tests/bin/wc]
 """,
-        "      1       2      10\n" if ON_WINDOWS else " 1  2 9 <stdin>\n",
+        " 1  2 10 <stdin>\n" if ON_WINDOWS else " 1  2 9 <stdin>\n",
         0,
     ),
     # test double  piping 'real' command
@@ -238,9 +223,9 @@ with open('tttt', 'w') as fp:
 with open('tttt', 'w') as fp:
     fp.write("Wow mom!\\n")
 
-![cat tttt | wc | wc]
+![python tests/bin/cat tttt | python tests/bin/wc | python tests/bin/wc]
 """,
-        "      1       3      24\n" if ON_WINDOWS else " 1  4 16 <stdin>\n",
+        " 1  4 18 <stdin>\n" if ON_WINDOWS else " 1  4 16 <stdin>\n",
         0,
     ),
     # test unthreadable alias (which should trigger a ProcPoxy call)
@@ -352,7 +337,7 @@ echo foo_@$(echo spam sausage)_bar
     (
         """
 echo Just the place for a snark. >tttt
-cat tttt
+python tests/bin/cat tttt
 """,
         "Just the place for a snark.\n",
         0,
@@ -472,6 +457,7 @@ a
 ]
 
 
+@skip_if_no_xonsh
 @pytest.mark.parametrize("case", ALL_PLATFORMS)
 def test_script(case):
     script, exp_out, exp_rtn = case
@@ -496,6 +482,7 @@ f o>e
 ]
 
 
+@skip_if_no_xonsh
 @pytest.mark.parametrize("case", ALL_PLATFORMS_STDERR)
 def test_script_stderr(case):
     script, exp_err, exp_rtn = case
@@ -504,6 +491,7 @@ def test_script_stderr(case):
     assert exp_rtn == rtn
 
 
+@skip_if_no_xonsh
 @skip_if_on_windows
 @pytest.mark.parametrize(
     "cmd, fmt, exp",
@@ -511,12 +499,14 @@ def test_script_stderr(case):
         ("pwd", None, lambda: os.getcwd() + "\n"),
         ("echo WORKING", None, "WORKING\n"),
         ("ls -f", lambda out: out.splitlines().sort(), os.listdir().sort()),
+        ("$FOO='foo' $BAR=2 xonsh -c r'echo -n $FOO$BAR'", None, "foo2",),
     ],
 )
 def test_single_command_no_windows(cmd, fmt, exp):
     check_run_xonsh(cmd, fmt, exp)
 
 
+@skip_if_no_xonsh
 def test_eof_syntax_error():
     """Ensures syntax errors for EOF appear on last line."""
     script = "x = 1\na = (1, 0\n"
@@ -525,6 +515,7 @@ def test_eof_syntax_error():
     assert ":2:0: EOF in multi-line statement" in err
 
 
+@skip_if_no_xonsh
 def test_open_quote_syntax_error():
     script = (
         "#!/usr/bin/env xonsh\n\n"
@@ -544,21 +535,25 @@ _bad_case = pytest.mark.skipif(
 )
 
 
+@skip_if_no_xonsh
 @_bad_case
 def test_printfile():
     check_run_xonsh("printfile.xsh", None, "printfile.xsh\n")
 
 
+@skip_if_no_xonsh
 @_bad_case
 def test_printname():
     check_run_xonsh("printfile.xsh", None, "printfile.xsh\n")
 
 
+@skip_if_no_xonsh
 @_bad_case
 def test_sourcefile():
     check_run_xonsh("printfile.xsh", None, "printfile.xsh\n")
 
 
+@skip_if_no_xonsh
 @_bad_case
 @pytest.mark.parametrize(
     "cmd, fmt, exp",
@@ -591,6 +586,7 @@ def test_subshells(cmd, fmt, exp):
     check_run_xonsh(cmd, fmt, exp)
 
 
+@skip_if_no_xonsh
 @skip_if_on_windows
 @pytest.mark.parametrize("cmd, exp", [("pwd", lambda: os.getcwd() + "\n")])
 def test_redirect_out_to_file(cmd, exp, tmpdir):
@@ -628,15 +624,17 @@ def test_xonsh_no_close_fds():
         assert "warning" not in out
 
 
+@skip_if_no_xonsh
 @pytest.mark.parametrize(
     "cmd, fmt, exp",
-    [("ls | wc", lambda x: x > "", True),],  # noqa E231 (black removes space)
+    [("cat tttt | wc", lambda x: x > "", True),],  # noqa E231 (black removes space)
 )
 def test_pipe_between_subprocs(cmd, fmt, exp):
     "verify pipe between subprocesses doesn't throw an exception"
     check_run_xonsh(cmd, fmt, exp)
 
 
+@skip_if_no_xonsh
 @skip_if_on_windows
 def test_negative_exit_codes_fail():
     # see issue 3309
@@ -646,6 +644,7 @@ def test_negative_exit_codes_fail():
     assert "OK" != err
 
 
+@skip_if_no_xonsh
 @pytest.mark.parametrize(
     "cmd, exp",
     [
@@ -670,6 +669,7 @@ aliases['echo'] = _echo
 
 
 # issue 3402
+@skip_if_no_xonsh
 @skip_if_on_windows
 @pytest.mark.parametrize(
     "cmd, exp_rtn",
@@ -685,6 +685,7 @@ def test_single_command_return_code(cmd, exp_rtn):
     assert rtn == exp_rtn
 
 
+@skip_if_no_xonsh
 @skip_if_on_msys
 @skip_if_on_windows
 @skip_if_on_darwin
