@@ -52,6 +52,8 @@ skip_if_on_darwin = pytest.mark.skipif(ON_DARWIN, reason="not Mac friendly")
 
 skip_if_on_travis = pytest.mark.skipif(ON_TRAVIS, reason="not Travis CI friendly")
 
+skip_if_pre_3_8 = pytest.mark.skipif(VER_FULL < (3, 8), reason="Python >= 3.8 feature")
+
 
 def sp(cmd):
     return subprocess.check_output(cmd, universal_newlines=True)
@@ -104,7 +106,10 @@ class DummyHistory:
 
 class DummyEnv(MutableMapping):
 
-    DEFAULTS = {"XONSH_DEBUG": 1, "XONSH_COLOR_STYLE": "default"}
+    DEFAULTS = {
+        "XONSH_DEBUG": 1,
+        "XONSH_COLOR_STYLE": "default",
+    }
 
     def __init__(self, *args, **kwargs):
         self._d = self.DEFAULTS.copy()
@@ -199,11 +204,9 @@ def nodes_equal(x, y):
         type(y),
     )
     if isinstance(x, (ast.Expr, ast.FunctionDef, ast.ClassDef)):
-        assert (
-            x.lineno == y.lineno
-        ), "Ast nodes do not have the same line number : %s != %s" % (
-            x.lineno,
-            y.lineno,
+        assert x.lineno == y.lineno, (
+            "Ast nodes do not have the same line number : %s != %s"
+            % (x.lineno, y.lineno,)
         )
         assert x.col_offset == y.col_offset, (
             "Ast nodes do not have the same column offset number : %s != %s"
