@@ -11,10 +11,11 @@ parser = argparse.ArgumentParser(description=program_description)
 parser.add_argument("env", nargs="*", default=[], metavar="ENV=value")
 parser.add_argument("--python", "-p", default="3.6", metavar="python_version")
 parser.add_argument("--pypy", default=None, metavar="pypy_version")
-parser.add_argument("--ptk", "-t", default="2.0.4", metavar="ptk_version")
+parser.add_argument("--ptk", "-t", default="3.0.7", metavar="ptk_version")
 parser.add_argument("--keep", action="store_true")
 parser.add_argument("--build", action="store_true")
 parser.add_argument("--command", "-c", default="xonsh", metavar="command")
+parser.add_argument("--pytest", action="store_true")
 
 args = parser.parse_args()
 
@@ -22,13 +23,16 @@ docker_script = """
 from python:{python_version}
 RUN pip install --upgrade pip && pip install \\
   prompt-toolkit=={ptk_version} \\
+  {pytest} \\
   pygments
 RUN mkdir /xonsh
 WORKDIR /xonsh
 ADD ./ ./
 RUN python setup.py install
 """.format(
-    python_version=args.python, ptk_version=args.ptk
+    python_version=args.python,
+    ptk_version=args.ptk,
+    pytest="pytest" if args.pytest else "",
 )
 
 if args.pypy:
@@ -37,13 +41,16 @@ from pypy:{python_version}
 RUN pypy3 -m ensurepip
 RUN pip install --upgrade pip && pip install \\
   prompt-toolkit=={ptk_version} \\
+  {pytest} \\
   pygments
 RUN mkdir /xonsh
 WORKDIR /xonsh
 ADD ./ ./
 RUN pypy3 setup.py install
     """.format(
-        python_version=args.pypy, ptk_version=args.ptk
+        python_version=args.pypy,
+        ptk_version=args.ptk,
+        pytest="pytest" if args.pytest else "",
     )
 
 print("Building and running Xonsh")
