@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 """Aliases for the xonsh shell."""
 import os
+import re
 import sys
-import shlex
 import inspect
 import argparse
 import builtins
@@ -19,17 +19,37 @@ from xonsh.platform import (
     ON_WINDOWS,
     ON_FREEBSD,
     ON_NETBSD,
+    ON_OPENBSD,
     ON_DRAGONFLY,
+    ON_POSIX,
+)
+from xonsh.tools import (
+    XonshError,
+    argvquote,
+    escape_windows_cmd_string,
+    to_bool,
+    swap_values,
+    strip_simple_quotes,
+    ALIAS_KWARG_NAMES,
+    unthreadable,
+    print_color,
 )
-from xonsh.tools import unthreadable, print_color
-from xonsh.replay import replay_main
 from xonsh.timings import timeit_alias
-from xonsh.tools import argvquote, escape_windows_cmd_string, to_bool, swap_values
 from xonsh.xontribs import xontribs_main
+from xonsh.ast import isexpression
 
 import xonsh.completers._aliases as xca
 import xonsh.history.main as xhm
 import xonsh.xoreutils.which as xxw
+from xonsh.xoreutils.umask import umask
+
+if ON_POSIX:
+    from xonsh.xoreutils.ulimit import ulimit
+
+
+@lazyobject
+def EXEC_ALIAS_RE():
+    return re.compile(r"@\(|\$\(|!\(|\$\[|!\[|\&\&|\|\||\s+and\s+|\s+or\s+|[>|<]")
 
 
 class Aliases(cabc.MutableMapping):
@@ -58,27 +78,19 @@ class Aliases(cabc.MutableMapping):
 
     def eval_alias(self, value, seen_tokens=frozenset(), acc_args=()):
         """
-        "Evaluates" the alias `value`, by recursively looking up the leftmost
+        "Evaluates" the alias ``value``, by recursively looking up the leftmost
         token and "expanding" if it's also an alias.
 
-        A value like ["cmd", "arg"] might transform like this:
-        > ["cmd", "arg"] -> ["ls", "-al", "arg"] -> callable()
-        where `cmd=ls -al` and `ls` is an alias with its value being a
+        A value like ``["cmd", "arg"]`` might transform like this:
+        ``> ["cmd", "arg"] -> ["ls", "-al", "arg"] -> callable()``
+        where ``cmd=ls -al`` and ``ls`` is an alias with its value being a
         callable.  The resulting callable will be "partially applied" with
-        ["-al", "arg"].
+        ``["-al", "arg"]``.
         """
         # Beware of mutability: default values for keyword args are evaluated
         # only once.
         if callable(value):
-            if acc_args:  # Partial application
-
-                def _alias(args, stdin=None):
-                    args = list(acc_args) + args
-                    return value(args, stdin=stdin)
-
-                return _alias
-            else:
-                return value
+            return partial_eval_alias(value, acc_args=acc_args)
         else:
             expand_path = builtins.__xonsh__.expand_path
             token, *rest = map(expand_path, value)
@@ -115,10 +127,37 @@ class Aliases(cabc.MutableMapping):
 
     def __setitem__(self, key, val):
         if isinstance(val, str):
-            self._raw[key] = shlex.split(val)
+            f = "<exec-alias:" + key + ">"
+            if EXEC_ALIAS_RE.search(val) is not None:
+                # We have a sub-command (e.g. $(cmd)) or IO redirect (e.g. >>)
+                self._raw[key] = ExecAlias(val, filename=f)
+            elif isexpression(val):
+                # expansion substitution
+                lexer = builtins.__xonsh__.execer.parser.lexer
+                self._raw[key] = list(map(strip_simple_quotes, lexer.split(val)))
+            else:
+                # need to exec alias
+                self._raw[key] = ExecAlias(val, filename=f)
         else:
             self._raw[key] = val
 
+    def _common_or(self, other):
+        new_dict = self._raw.copy()
+        for key in dict(other):
+            new_dict[key] = other[key]
+        return Aliases(new_dict)
+
+    def __or__(self, other):
+        return self._common_or(other)
+
+    def __ror__(self, other):
+        return self._common_or(other)
+
+    def __ior__(self, other):
+        for key in dict(other):
+            self[key] = other[key]
+        return self
+
     def __delitem__(self, key):
         del self._raw[key]
 
@@ -150,6 +189,161 @@ class Aliases(cabc.MutableMapping):
                 p.pretty(dict(self))
 
 
+class ExecAlias:
+    """Provides a callable alias for xonsh source code."""
+
+    def __init__(self, src, filename="<exec-alias>"):
+        """
+        Parameters
+        ----------
+        src : str
+            Source code that will be
+        """
+        self.src = src if src.endswith("\n") else src + "\n"
+        self.filename = filename
+
+    def __call__(
+        self, args, stdin=None, stdout=None, stderr=None, spec=None, stack=None
+    ):
+        execer = builtins.__xonsh__.execer
+        frame = stack[0][0]  # execute as though we are at the call site
+        execer.exec(
+            self.src, glbs=frame.f_globals, locs=frame.f_locals, filename=self.filename
+        )
+
+    def __repr__(self):
+        return "ExecAlias({0!r}, filename={1!r})".format(self.src, self.filename)
+
+
+class PartialEvalAliasBase:
+    """Partially evaluated alias."""
+
+    def __init__(self, f, acc_args=()):
+        """
+        Parameters
+        ----------
+        f : callable
+            A function to dispatch to.
+        acc_args : sequence of strings, optional
+            Additional arguments to prepent to the argument list passed in
+            when the alias is called.
+        """
+        self.f = f
+        self.acc_args = acc_args
+        self.__name__ = getattr(f, "__name__", self.__class__.__name__)
+
+    def __call__(
+        self, args, stdin=None, stdout=None, stderr=None, spec=None, stack=None
+    ):
+        args = list(self.acc_args) + args
+        return self.f(args, stdin, stdout, stderr, spec, stack)
+
+    def __repr__(self):
+        return "{name}({f!r}, acc_args={acc_args!r})".format(
+            name=self.__class__.__name__, f=self.f, acc_args=self.acc_args
+        )
+
+
+class PartialEvalAlias0(PartialEvalAliasBase):
+    def __call__(
+        self, args, stdin=None, stdout=None, stderr=None, spec=None, stack=None
+    ):
+        args = list(self.acc_args) + args
+        if args:
+            msg = "callable alias {f!r} takes no arguments, but {args!f} provided. "
+            msg += "Of these {acc_args!r} were partially applied."
+            raise XonshError(msg.format(f=self.f, args=args, acc_args=self.acc_args))
+        return self.f()
+
+
+class PartialEvalAlias1(PartialEvalAliasBase):
+    def __call__(
+        self, args, stdin=None, stdout=None, stderr=None, spec=None, stack=None
+    ):
+        args = list(self.acc_args) + args
+        return self.f(args)
+
+
+class PartialEvalAlias2(PartialEvalAliasBase):
+    def __call__(
+        self, args, stdin=None, stdout=None, stderr=None, spec=None, stack=None
+    ):
+        args = list(self.acc_args) + args
+        return self.f(args, stdin)
+
+
+class PartialEvalAlias3(PartialEvalAliasBase):
+    def __call__(
+        self, args, stdin=None, stdout=None, stderr=None, spec=None, stack=None
+    ):
+        args = list(self.acc_args) + args
+        return self.f(args, stdin, stdout)
+
+
+class PartialEvalAlias4(PartialEvalAliasBase):
+    def __call__(
+        self, args, stdin=None, stdout=None, stderr=None, spec=None, stack=None
+    ):
+        args = list(self.acc_args) + args
+        return self.f(args, stdin, stdout, stderr)
+
+
+class PartialEvalAlias5(PartialEvalAliasBase):
+    def __call__(
+        self, args, stdin=None, stdout=None, stderr=None, spec=None, stack=None
+    ):
+        args = list(self.acc_args) + args
+        return self.f(args, stdin, stdout, stderr, spec)
+
+
+class PartialEvalAlias6(PartialEvalAliasBase):
+    def __call__(
+        self, args, stdin=None, stdout=None, stderr=None, spec=None, stack=None
+    ):
+        args = list(self.acc_args) + args
+        return self.f(args, stdin, stdout, stderr, spec, stack)
+
+
+PARTIAL_EVAL_ALIASES = (
+    PartialEvalAlias0,
+    PartialEvalAlias1,
+    PartialEvalAlias2,
+    PartialEvalAlias3,
+    PartialEvalAlias4,
+    PartialEvalAlias5,
+    PartialEvalAlias6,
+)
+
+
+def partial_eval_alias(f, acc_args=()):
+    """Dispatches the appropriate eval alias based on the number of args to the original callable alias
+    and how many arguments to apply.
+    """
+    # no partial needed if no extra args
+    if not acc_args:
+        return f
+    # need to dispatch
+    numargs = 0
+    for name, param in inspect.signature(f).parameters.items():
+        if (
+            param.kind == param.POSITIONAL_ONLY
+            or param.kind == param.POSITIONAL_OR_KEYWORD
+        ):
+            numargs += 1
+        elif name in ALIAS_KWARG_NAMES and param.kind == param.KEYWORD_ONLY:
+            numargs += 1
+    if numargs < 7:
+        return PARTIAL_EVAL_ALIASES[numargs](f, acc_args=acc_args)
+    else:
+        e = "Expected proxy with 6 or fewer arguments for {}, not {}"
+        raise XonshError(e.format(", ".join(ALIAS_KWARG_NAMES), numargs))
+
+
+#
+# Actual aliases below
+#
+
+
 def xonsh_exit(args, stdin=None):
     """Sends signal to exit shell."""
     if not clean_jobs():
@@ -364,6 +558,7 @@ def source_foreign(args, stdin=None, stdout=None, stderr=None):
             print(msg.format(k, ns.shell), file=stderr)
 
 
+@unthreadable
 def source_alias(args, stdin=None):
     """Executes the contents of the provided files in the current context.
     If sourced file isn't found in cwd, search for file along $PATH to source
@@ -381,7 +576,7 @@ def source_alias(args, stdin=None):
                     print("source: {}: No such file".format(fname), file=sys.stderr)
                 if i == 0:
                     raise RuntimeError(
-                        "must source at least one file, " + fname + "does not exist."
+                        "must source at least one file, " + fname + " does not exist."
                     )
                 break
         _, fext = os.path.splitext(fpath)
@@ -404,10 +599,10 @@ def source_alias(args, stdin=None):
             except Exception:
                 print_color(
                     "{RED}You may be attempting to source non-xonsh file! "
-                    "{NO_COLOR}If you are trying to source a file in "
+                    "{RESET}If you are trying to source a file in "
                     "another language, then please use the appropriate "
                     "source command. For example, {GREEN}source-bash "
-                    "script.sh{NO_COLOR}",
+                    "script.sh{RESET}",
                     file=sys.stderr,
                 )
                 raise
@@ -435,7 +630,7 @@ def source_cmd(args, stdin=None):
 
 
 def xexec(args, stdin=None):
-    """exec [-h|--help] command [args...]
+    """exec [-h|--help] [-cl] [-a name] command [args...]
 
     exec (also aliased as xexec) uses the os.execvpe() function to
     replace the xonsh process with the specified program. This provides
@@ -445,6 +640,13 @@ def xexec(args, stdin=None):
         bash $
 
     The '-h' and '--help' options print this message and exit.
+    If the '-l' option is supplied, the shell places a dash at the
+    beginning of the zeroth argument passed to command to simulate login
+    shell.
+    The '-c' option causes command to be executed with an empty environment.
+    If '-a' is supplied, the shell passes name as the zeroth argument
+    to the executed command.
+
 
     Notes
     -----
@@ -458,18 +660,40 @@ def xexec(args, stdin=None):
     """
     if len(args) == 0:
         return (None, "xonsh: exec: no args specified\n", 1)
-    elif args[0] == "-h" or args[0] == "--help":
+
+    parser = argparse.ArgumentParser(add_help=False)
+    parser.add_argument("-h", "--help", action="store_true")
+    parser.add_argument("-l", dest="login", action="store_true")
+    parser.add_argument("-c", dest="clean", action="store_true")
+    parser.add_argument("-a", dest="name", nargs="?")
+    parser.add_argument("command", nargs=argparse.REMAINDER)
+    args = parser.parse_args(args)
+
+    if args.help:
         return inspect.getdoc(xexec)
-    else:
+
+    if len(args.command) == 0:
+        return (None, "xonsh: exec: no command specified\n", 1)
+
+    command = args.command[0]
+    if args.name is not None:
+        args.command[0] = args.name
+    if args.login:
+        args.command[0] = "-{}".format(args.command[0])
+
+    denv = {}
+    if not args.clean:
         denv = builtins.__xonsh__.env.detype()
-        try:
-            os.execvpe(args[0], args, denv)
-        except FileNotFoundError as e:
-            return (
-                None,
-                "xonsh: exec: file not found: {}: {}" "\n".format(e.args[1], args[0]),
-                1,
-            )
+
+    try:
+        os.execvpe(command, args.command, denv)
+    except FileNotFoundError as e:
+        return (
+            None,
+            "xonsh: exec: file not found: {}: {}"
+            "\n".format(e.args[1], args.command[0]),
+            1,
+        )
 
 
 class AWitchAWitch(argparse.Action):
@@ -517,7 +741,8 @@ def showcmd(args, stdin=None):
     optional arguments:
       -h, --help            show this help message and exit
 
-    example:
+    Example:
+    -------
       >>> showcmd echo $USER can't hear "the sea"
       ['echo', 'I', "can't", 'hear', 'the sea']
     """
@@ -573,9 +798,9 @@ def make_default_aliases():
         "source-cmd": source_cmd,
         "source-foreign": source_foreign,
         "history": xhm.history_main,
-        "replay": replay_main,
         "trace": trace,
         "timeit": timeit_alias,
+        "umask": umask,
         "xonfig": xonfig,
         "scp-resume": ["rsync", "--partial", "-h", "--progress", "--rsh=ssh"],
         "showcmd": showcmd,
@@ -586,6 +811,8 @@ def make_default_aliases():
         "xpip": detect_xpip_alias(),
         "xonsh-reset": xonsh_reset,
     }
+    if ON_POSIX:
+        default_aliases["ulimit"] = ulimit
     if ON_WINDOWS:
         # Borrow builtin commands from cmd.exe.
         windows_cmd_aliases = {
@@ -647,6 +874,8 @@ def make_default_aliases():
         default_aliases["grep"] = ["grep", "--color=auto"]
         default_aliases["egrep"] = ["egrep", "--color=auto"]
         default_aliases["fgrep"] = ["fgrep", "--color=auto"]
+    elif ON_OPENBSD:
+        pass
     else:
         default_aliases["grep"] = ["grep", "--color=auto"]
         default_aliases["egrep"] = ["egrep", "--color=auto"]
