@@ -3,19 +3,23 @@
 import os
 import sys
 import time
-import json
 import builtins
 import collections
 import threading
 import collections.abc as cabc
 
+try:
+    import ujson as json
+except ImportError:
+    import json  # type: ignore
+
 from xonsh.history.base import History
 import xonsh.tools as xt
 import xonsh.lazyjson as xlj
 import xonsh.xoreutils.uptime as uptime
 
 
-def _xhj_gc_commands_to_rmfiles(hsize, files) -> ([], int):
+def _xhj_gc_commands_to_rmfiles(hsize, files):
     """Return number of units and list of history files to remove to get under the limit,
 
     Parameters:
@@ -162,7 +166,7 @@ class JsonHistoryGC(threading.Thread):
         else:
             print(
                 f"Warning: History garbage collection would discard more history ({size_over} {units}) than it would keep ({hsize}).\n"
-                "Not removing any history for now. Either increase your limit ($XONSH_HIST_SIZE), or run `history gc --force`."
+                "Not removing any history for now. Either increase your limit ($XONSH_HISTORY_SIZE), or run `history gc --force`."
             )
 
     def files(self, only_unlocked=False):
@@ -180,6 +184,7 @@ class JsonHistoryGC(threading.Thread):
         boot = uptime.boottime()
         fs = _xhj_get_history_files(sort=False)
         files = []
+        time_start = time.time()
         for f in fs:
             try:
                 cur_file_size = os.path.getsize(f)
@@ -188,7 +193,7 @@ class JsonHistoryGC(threading.Thread):
                     files.append((os.path.getmtime(f), 0, f, cur_file_size))
                     continue
                 lj = xlj.LazyJSON(f, reopen=False)
-                if lj["locked"] and lj["ts"][0] < boot:
+                if lj.get("locked", False) and lj["ts"][0] < boot:
                     # computer was rebooted between when this history was created
                     # and now and so this history should be unlocked.
                     hist = lj.load()
@@ -197,20 +202,21 @@ class JsonHistoryGC(threading.Thread):
                     with open(f, "w", newline="\n") as fp:
                         xlj.ljdump(hist, fp, sort_keys=True)
                     lj = xlj.LazyJSON(f, reopen=False)
-                if only_unlocked and lj["locked"]:
+                if only_unlocked and lj.get("locked", False):
                     continue
                 # info: file size, closing timestamp, number of commands, filename
+                ts = lj.get("ts", (0.0, None))
                 files.append(
-                    (
-                        lj["ts"][1] or lj["ts"][0],
-                        len(lj.sizes["cmds"]) - 1,
-                        f,
-                        cur_file_size,
-                    ),
+                    (ts[1] or ts[0], len(lj.sizes["cmds"]) - 1, f, cur_file_size)
                 )
                 lj.close()
                 if xonsh_debug:
-                    print(f"... Enumerated {len(files):7d} history files.\r", end="")
+                    time_lag = time.time() - time_start
+                    print(
+                        f"[history.{json.__name__}] Enumerated {len(files):,d} history files for {time_lag:0.4f}s.\r",
+                        end="",
+                        file=sys.stderr,
+                    )
             except (IOError, OSError, ValueError):
                 continue
         files.sort()  # this sorts by elements of the tuple,
@@ -266,6 +272,7 @@ class JsonHistoryFlusher(threading.Thread):
                 if self.skip is not None:
                     self.skip(1)
                 continue
+
             cmds.append(cmd)
             last_inp = cmd["inp"]
         with open(self.filename, "r", newline="\n") as f:
@@ -307,6 +314,9 @@ class JsonCommandField(cabc.Sequence):
         return len(self.hist)
 
     def __getitem__(self, key):
+        if not self.hist.remember_history:
+            return ""
+
         size = len(self)
         if isinstance(key, slice):
             return [self[i] for i in range(*key.indices(size))]
@@ -405,14 +415,30 @@ class JsonHistory(History):
             added to the history list. It should contain the keys ``inp``,
             ``rtn`` and ``ts``. These key names mirror the same names defined
             as instance variables in the ``HistoryEntry`` class.
+            Additionally, an optional key ``spc`` may be present which will
+            affect commands from being stored if ignorespace is in $HISTCONTROL.
 
         Returns
         -------
         hf : JsonHistoryFlusher or None
             The thread that was spawned to flush history
         """
+        if not self.remember_history:
+            return
+
+        opts = builtins.__xonsh__.env.get("HISTCONTROL")
+        skipped_by_ignore_space = "ignorespace" in opts and cmd.get("spc")
+        if skipped_by_ignore_space:
+            return None
+
         self.buffer.append(cmd)
         self._len += 1  # must come before flushing
+
+        try:
+            del cmd["spc"]
+        except KeyError:
+            pass
+
         if len(self.buffer) >= self.buffersize:
             hf = self.flush()
         else:
@@ -433,6 +459,7 @@ class JsonHistory(History):
         hf : JsonHistoryFlusher or None
             The thread that was spawned to flush history
         """
+        # Implicitly covers case of self.remember_history being False.
         if len(self.buffer) == 0:
             return
 
@@ -475,7 +502,7 @@ class JsonHistory(History):
                 continue
             try:
                 commands = json_file.load()["cmds"]
-            except json.decoder.JSONDecodeError:
+            except (json.decoder.JSONDecodeError, ValueError):
                 # file is corrupted somehow
                 if builtins.__xonsh__.env.get("XONSH_DEBUG") > 0:
                     msg = "xonsh history file {0!r} is not valid JSON"
@@ -506,3 +533,18 @@ class JsonHistory(History):
         if blocking:
             while self.gc.is_alive():  # while waiting for gc.
                 time.sleep(0.1)  # don't monopolize the thread (or Python GIL?)
+
+    def clear(self):
+        """Clears the current session's history from both memory and disk."""
+
+        # Wipe history from memory. Keep sessionid and other metadata.
+        self.buffer = []
+        self.tss = JsonCommandField("ts", self)
+        self.inps = JsonCommandField("inp", self)
+        self.outs = JsonCommandField("out", self)
+        self.rtns = JsonCommandField("rtn", self)
+        self._len = 0
+        self._skipped = 0
+
+        # Flush empty history object to disk, overwriting previous data.
+        self.flush()
