@@ -143,6 +143,13 @@ def pygments_version_info():
         return None
 
 
+def use_vended_prompt_toolkit():
+    """ Unload any prompt_toolkit libraries and add vended version to sys.path """
+    for mod in (mod for mod in list(sys.modules) if mod.startswith("prompt_toolkit")):
+        del sys.modules[mod]
+    sys.path.insert(0, str(pathlib.Path(__file__).with_name("vended_ptk").resolve()))
+
+
 @functools.lru_cache(1)
 def has_prompt_toolkit():
     """Tests if the `prompt_toolkit` is available."""
@@ -170,13 +177,13 @@ def ptk_version_info():
         return None
 
 
-minimum_required_ptk_version = (2, 0)
+minimum_required_ptk_version = (2, 0, 0)
 """Minimum version of prompt-toolkit supported by Xonsh"""
 
 
 @functools.lru_cache(1)
 def ptk_above_min_supported():
-    return ptk_version_info() and ptk_version_info()[:2] >= minimum_required_ptk_version
+    return ptk_version_info() and ptk_version_info() >= minimum_required_ptk_version
 
 
 @functools.lru_cache(1)
@@ -201,10 +208,8 @@ def ptk_below_max_supported():
 def best_shell_type():
     if builtins.__xonsh__.env.get("TERM", "") == "dumb":
         return "dumb"
-    elif ON_WINDOWS or has_prompt_toolkit():
-        return "prompt_toolkit"
     else:
-        return "readline"
+        return "prompt_toolkit"
 
 
 @functools.lru_cache(1)
@@ -388,10 +393,10 @@ def windows_bash_command():
 if ON_WINDOWS:
 
     class OSEnvironCasePreserving(cabc.MutableMapping):
-        """ Case-preserving wrapper for os.environ on Windows.
-            It uses nt.environ to get the correct cased keys on
-            initialization. It also preserves the case of any variables
-            add after initialization.
+        """Case-preserving wrapper for os.environ on Windows.
+        It uses nt.environ to get the correct cased keys on
+        initialization. It also preserves the case of any variables
+        add after initialization.
         """
 
         def __init__(self):
@@ -400,8 +405,8 @@ if ON_WINDOWS:
             self._upperkeys = dict((k.upper(), k) for k in nt.environ)
 
         def _sync(self):
-            """ Ensure that the case sensitive map of the keys are
-                in sync with os.environ
+            """Ensure that the case sensitive map of the keys are
+            in sync with os.environ
             """
             envkeys = set(os.environ.keys())
             for key in envkeys.difference(self._upperkeys):
@@ -545,6 +550,11 @@ def LIBC():
         libc = ctypes.CDLL("cygwin1.dll")
     elif ON_MSYS:
         libc = ctypes.CDLL("msys-2.0.dll")
+    elif ON_FREEBSD:
+        try:
+            libc = ctypes.CDLL("libc.so.7")
+        except OSError:
+            libc = None
     elif ON_BSD:
         try:
             libc = ctypes.CDLL("libc.so")
