@@ -7,9 +7,6 @@ version 2), which provides tokenization help for Python programs.
 It is modified to properly tokenize xonsh code, including backtick regex
 path and several xonsh-specific operators.
 
-A few pieces of this file are specific to the version of Python being used.
-To find these pieces, search the PY35.
-
 Original file credits:
    __author__ = 'Ka-Ping Yee <ping@lfw.org>'
    __credits__ = ('GvR, ESR, Tim Peters, Thomas Wouters, Fred Drake, '
@@ -81,10 +78,15 @@ from token import (
     VBAREQUAL,
     tok_name,
 )
+import typing as tp
 
 from xonsh.lazyasd import LazyObject
 from xonsh.platform import PYTHON_VERSION_INFO
 
+HAS_WALRUS = PYTHON_VERSION_INFO > (3, 8)
+if HAS_WALRUS:
+    from token import COLONEQUAL  # type:ignore
+
 cookie_re = LazyObject(
     lambda: re.compile(r"^[ \t\f]*#.*coding[:=][ \t]*([-\w.]+)", re.ASCII),
     globals(),
@@ -98,7 +100,7 @@ blank_re = LazyObject(
 # token modifications
 #
 tok_name = tok_name.copy()
-__all__ = token.__all__ + [
+__all__ = token.__all__ + [  # type:ignore
     "COMMENT",
     "tokenize",
     "detect_encoding",
@@ -113,19 +115,19 @@ __all__ = token.__all__ + [
     "DOLLARNAME",
     "IOREDIRECT",
 ]
-HAS_ASYNC = (3, 5, 0) <= PYTHON_VERSION_INFO < (3, 7, 0)
+HAS_ASYNC = PYTHON_VERSION_INFO < (3, 7, 0)
 if HAS_ASYNC:
-    ASYNC = token.ASYNC
-    AWAIT = token.AWAIT
+    ASYNC = token.ASYNC  # type:ignore
+    AWAIT = token.AWAIT  # type:ignore
     ADDSPACE_TOKS = (NAME, NUMBER, ASYNC, AWAIT)
 else:
-    ADDSPACE_TOKS = (NAME, NUMBER)
+    ADDSPACE_TOKS = (NAME, NUMBER)  # type:ignore
 del token  # must clean up token
-PY35 = (3, 5, 0) <= PYTHON_VERSION_INFO
-AUGASSIGN_OPS = r"[+\-*/%&@|^=<>]=?"
-if not PY35:
-    AUGASSIGN_OPS = AUGASSIGN_OPS.replace("@", "")
 
+if HAS_WALRUS:
+    AUGASSIGN_OPS = r"[+\-*/%&@|^=<>:]=?"
+else:
+    AUGASSIGN_OPS = r"[+\-*/%&@|^=<>]=?"
 
 COMMENT = N_TOKENS
 tok_name[COMMENT] = "COMMENT"
@@ -175,7 +177,7 @@ for v in _xonsh_tokens.values():
     __all__.append(v)
 del _glbs, v
 
-EXACT_TOKEN_TYPES = {
+EXACT_TOKEN_TYPES: tp.Dict[str, tp.Union[str, int]] = {
     "(": LPAR,
     ")": RPAR,
     "[": LSQB,
@@ -220,6 +222,8 @@ EXACT_TOKEN_TYPES = {
     "//=": DOUBLESLASHEQUAL,
     "@": AT,
 }
+if HAS_WALRUS:
+    EXACT_TOKEN_TYPES[":="] = COLONEQUAL
 
 EXACT_TOKEN_TYPES.update(_xonsh_tokens)
 
@@ -326,10 +330,12 @@ _redir_map = (
     "1>2",
 )
 IORedirect = group(group(*_redir_map), "{}>>?".format(group(*_redir_names)))
-_redir_check = set(_redir_map)
-_redir_check = {"{}>".format(i) for i in _redir_names}.union(_redir_check)
-_redir_check = {"{}>>".format(i) for i in _redir_names}.union(_redir_check)
-_redir_check = frozenset(_redir_check)
+
+_redir_check_0 = set(_redir_map)
+_redir_check_1 = {"{}>".format(i) for i in _redir_names}.union(_redir_check_0)
+_redir_check_2 = {"{}>>".format(i) for i in _redir_names}.union(_redir_check_1)
+_redir_check = frozenset(_redir_check_2)
+
 Operator = group(
     r"\*\*=?",
     r">>=?",
